@@ -1,6 +1,8 @@
 /*
- * Copyright (C) 2002 Samuel Hocevar <sam@zoy.org>,
- *                    Håkan Hjort <d95hjort@dtek.chalmers.se>
+ * Copyright (C) 2002-2025 Samuel Hocevar <sam@zoy.org>,
+ *                         Håkan Hjort <d95hjort@dtek.chalmers.se>
+ *                         Jean-Baptiste Kempf <jb@videolan.org>
+ *                         VideoLAN
  *
  * This file is part of libdvdread.
  *
@@ -20,10 +22,10 @@
  */
 
 #include "config.h"                  /* Required for HAVE_DVDCSS_DVDCSS_H */
-#include <stdio.h>                               /* fprintf */
-#include <stdlib.h>                              /* free */
-#include <fcntl.h>                               /* open */
-#include <unistd.h>                              /* lseek */
+#include <stdio.h>                   /* fprintf */
+#include <stdlib.h>                  /* free */
+#include <fcntl.h>                   /* open */
+#include <unistd.h>                  /* lseek */
 #include <string.h>                  /* strerror */
 #include <errno.h>
 #include <assert.h>
@@ -274,7 +276,7 @@ static int cpxm_init(dvd_input_t dev, uint8_t* p_mkb )
 {
   return DVDcpxm_init(dev->dvdcss, p_mkb);
 }
-#endif
+#endif /* CPXM */
 
 /**
  * initialize and open a DVD device or file.
@@ -288,8 +290,7 @@ static dvd_input_t file_open(void *priv, dvd_logger_cb *logcb,
   /* Allocate the library structure */
   dev = dvd_input_New(priv, logcb);
   if(dev == NULL) {
-    DVDReadLog(priv, logcb, DVD_LOGGER_LEVEL_ERROR,
-               "Could not allocate memory.");
+    DVDReadLog(priv, logcb, DVD_LOGGER_LEVEL_ERROR, "Could not allocate memory.");
     return NULL;
   }
 
@@ -297,7 +298,7 @@ static dvd_input_t file_open(void *priv, dvd_logger_cb *logcb,
   if (stream_cb) {
     if (!stream_cb->pf_read || !stream_cb->pf_seek) {
       DVDReadLog(priv, logcb, DVD_LOGGER_LEVEL_ERROR,
-                 "Stream callback provided but lacks of pf_read or pf_seek methods.");
+              "Stream callback provided but lacks of pf_read or pf_seek methods.");
       free(dev);
       return NULL;
     }
@@ -396,7 +397,7 @@ static int file_title(dvd_input_t dev UNUSED, int block UNUSED)
  * read data from the device.
  */
 static int file_read(dvd_input_t dev, void *buffer, int blocks,
-		     int flags UNUSED)
+                     int flags UNUSED)
 {
   size_t len, bytes, blocks_read;
 
@@ -471,8 +472,8 @@ int dvdinput_setup(void *priv, dvd_logger_cb *logcb, dvd_type_t dvda_flag)
 #ifdef HAVE_DVDCSS_DVDCPXM_H
   /* linking to libdvdcss */
   dvdcss_library = &dvdcss_library;  /* Give it some value != NULL */
-#elif defined( HAVE_DVDCSS_DVDCSS_H)
-  if (dvda_flag ==DVD_V)
+#elif defined( HAVE_DVDCSS_DVDCSS_H )
+  if (dvda_flag == DVD_V )
     dvdcss_library = &dvdcss_library;  /* Give it some value != NULL */
 #else
   /* dlopening libdvdcss */
@@ -492,6 +493,13 @@ int dvdinput_setup(void *priv, dvd_logger_cb *logcb, dvd_type_t dvda_flag)
 #define WIDEN_(x) L ## x
 #define WIDEN(x) WIDEN_(x)
 
+#ifdef __OS2__
+#define U_S "_"
+#else
+#define U_S
+#endif
+
+/* Actually dlopen */
 #if defined(_WIN32)
 # if WINAPI_FAMILY_PARTITION(WINAPI_PARTITION_DESKTOP)
     UINT em;
@@ -515,14 +523,10 @@ int dvdinput_setup(void *priv, dvd_logger_cb *logcb, dvd_type_t dvda_flag)
   dvdcss_library = dlopen(CSS_LIB, RTLD_LAZY);
 #endif
 
+  /* Locate the functions, DVD_V or DVD_A */
   if(dvdcss_library != NULL) {
-#ifdef __OS2__
-#define U_S "_"
-#else
-#define U_S
-#endif
     /* functions should have the same template*/
-    switch(dvda_flag){
+    switch(dvda_flag) {
       case DVD_V:
       /* hybrid discs encrypt video tracks with css*/
         DVDcss_open_stream = (dvdcss_t (*)(void *, dvdcss_stream_cb *))
@@ -539,7 +543,14 @@ int dvdinput_setup(void *priv, dvd_logger_cb *logcb, dvd_type_t dvda_flag)
           DVDReadLog(priv, logcb, DVD_LOGGER_LEVEL_ERROR,
                      "Old (pre-0.0.2) version of libdvdcss found. "
                     "libdvdread: You should get the latest version from "
-                    "http://www.videolan.org/" );
+                    "https://www.videolan.org/" );
+        } else if( ( !DVDcss_open || !DVDcss_close || !DVDcss_seek
+            || !DVDcss_read ) &&  dvda_flag == DVD_V ) {
+          DVDReadLog(priv, logcb, DVD_LOGGER_LEVEL_ERROR,
+                     "Missing symbols in %s, "
+                    "this shouldn't happen !", CSS_LIB);
+          dlclose(dvdcss_library);
+          dvdcss_library = NULL;
         }
       break;
       case DVD_A:
@@ -556,30 +567,24 @@ int dvdinput_setup(void *priv, dvd_logger_cb *logcb, dvd_type_t dvda_flag)
           dlsym(dvdcss_library, U_S "dvdcpxm_read");
         DVDcpxm_init = (int (*)(dvdcss_t, uint8_t *p_mkb))
           dlsym(dvdcss_library, U_S "dvdcpxm_init");
-#else 
+
+        if( ( !DVDcpxm_open || !DVDcpxm_close || !DVDcpxm_seek
+                  || !DVDcpxm_read || !DVDcpxm_init ) && dvda_flag == DVD_A ) {
+          DVDReadLog(priv, logcb, DVD_LOGGER_LEVEL_ERROR,
+                     "Missing symbols in %s, "
+                    "DVD-Audio support not present in libdvdcss", CSS_LIB);
+          dlclose(dvdcss_library);
+          dvdcss_library = NULL;
+        }
+#else /* We are trying to open a DVD-Audio, but we don't have the DVDcss CPXM */
+        DVDReadLog(priv, logcb, DVD_LOGGER_LEVEL_ERROR,
+                "DVD-Audio headers not present, update the DVDCSS library");
         dlclose(dvdcss_library);
         dvdcss_library = NULL;
+#endif
       break;
-#endif
     }
-  } else if(!DVDcss_open || !DVDcss_close || !DVDcss_seek
-            || !DVDcss_read) {
-    DVDReadLog(priv, logcb, DVD_LOGGER_LEVEL_ERROR,
-               "Missing symbols in %s, "
-              "this shouldn't happen !", CSS_LIB);
-    dlclose(dvdcss_library);
-    dvdcss_library = NULL;
   }
-#ifdef HAVE_DVDCSS_DVDCPXM_H
-  } else if( ( !DVDcpxm_open || !DVDcpxm_close || !DVDcpxm_seek
-            || !DVDcpxm_read || !DVDcpxm_init ) && dvda_flag == DVD_A ) {
-    DVDReadLog(priv, logcb, DVD_LOGGER_LEVEL_ERROR,
-               "Missing symbols in %s, "
-              "DVD-Audio support not present in libdvdcss", CSS_LIB);
-    dlclose(dvdcss_library);
-    dvdcss_library = NULL;
-  }
-#endif
 #endif /* HAVE_DVDCSS_DVDCSS_H */
 
   if(dvdcss_library != NULL) {
@@ -604,11 +609,10 @@ int dvdinput_setup(void *priv, dvd_logger_cb *logcb, dvd_type_t dvda_flag)
         dvdinput_open  = css_open;
         dvdinput_close = cpxm_close;
         dvdinput_seek  = cpxm_seek;
-        /* cpxm title is just seek */
-        dvdinput_title = cpxm_seek;
+        dvdinput_title = cpxm_seek; /* cpxm title is just seek */
         dvdinput_read  = cpxm_read;
         dvdinput_init  = cpxm_init;
-#else 
+#else
         assert(!"libdvdcss compiled without DVD-Audio (CPXM) support");
 #endif
         break;
