@@ -28,10 +28,13 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
+#include "config.h"
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <strings.h>
+#ifndef _MSC_VER
+# include <strings.h>
+#endif
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -43,16 +46,16 @@
 #include "dvdread/dvd_udf.h"
 
 /* It's required to either fail or deliver all the blocks asked for. */
-static int DVDReadLBUDF( dvd_reader_t *device, uint32_t lb_number,
+static int DVDReadLBUDF( dvd_reader_t *ctx, uint32_t lb_number,
                          size_t block_count, unsigned char *data,
                          int encrypted )
 {
-  int ret;
   size_t count = block_count;
 
   while(count > 0) {
+    int ret;
 
-    ret = UDFReadBlocksRaw(device, lb_number, count, data, encrypted);
+    ret = InternalUDFReadBlocksRaw(ctx, lb_number, count, data + DVD_VIDEO_LB_LEN * (block_count - count), encrypted);
 
     if(ret <= 0) {
       /* One of the reads failed or nothing more to read, too bad.
@@ -74,7 +77,6 @@ static int DVDReadLBUDF( dvd_reader_t *device, uint32_t lb_number,
 
 struct Partition {
   int valid;
-  char VolumeDesc[128];
   uint16_t Flags;
   uint16_t Number;
   char Contents[32];
@@ -155,16 +157,16 @@ void FreeUDFCache(void *cache)
 }
 
 
-static int GetUDFCache(dvd_reader_t *device, UDFCacheType type,
+static int GetUDFCache(dvd_reader_t *ctx, UDFCacheType type,
                        uint32_t nr, void *data)
 {
   int n;
   struct udf_cache *c;
 
-  if(DVDUDFCacheLevel(device, -1) <= 0)
+  if(DVDUDFCacheLevel(ctx, -1) <= 0)
     return 0;
 
-  c = (struct udf_cache *)GetUDFCacheHandle(device);
+  c = (struct udf_cache *)GetUDFCacheHandle(ctx);
 
   if(c == NULL)
     return 0;
@@ -217,24 +219,24 @@ static int GetUDFCache(dvd_reader_t *device, UDFCacheType type,
   return 0;
 }
 
-static int SetUDFCache(dvd_reader_t *device, UDFCacheType type,
+static int SetUDFCache(dvd_reader_t *ctx, UDFCacheType type,
                        uint32_t nr, void *data)
 {
   int n;
   struct udf_cache *c;
   void *tmp;
 
-  if(DVDUDFCacheLevel(device, -1) <= 0)
+  if(DVDUDFCacheLevel(ctx, -1) <= 0)
     return 0;
 
-  c = (struct udf_cache *)GetUDFCacheHandle(device);
+  c = (struct udf_cache *)GetUDFCacheHandle(ctx);
 
   if(c == NULL) {
     c = calloc(1, sizeof(struct udf_cache));
-    /* fprintf(stderr, "calloc: %d\n", sizeof(struct udf_cache)); */
+    /* Log3(ctx, "calloc: %d", sizeof(struct udf_cache)); */
     if(c == NULL)
       return 0;
-    SetUDFCacheHandle(device, c);
+    SetUDFCacheHandle(ctx, c);
   }
 
 
@@ -268,7 +270,7 @@ static int SetUDFCache(dvd_reader_t *device, UDFCacheType type,
     c->lb_num++;
     tmp = realloc(c->lbs, c->lb_num * sizeof(struct lbudf));
     /*
-    fprintf(stderr, "realloc lb: %d * %d = %d\n",
+    Log3(ctx, "realloc lb: %d * %d = %d",
     c->lb_num, sizeof(struct lbudf),
     c->lb_num * sizeof(struct lbudf));
     */
@@ -294,7 +296,7 @@ static int SetUDFCache(dvd_reader_t *device, UDFCacheType type,
     c->map_num++;
     tmp = realloc(c->maps, c->map_num * sizeof(struct icbmap));
     /*
-    fprintf(stderr, "realloc maps: %d * %d = %d\n",
+    Log3(ctx, "realloc maps: %d * %d = %d\n",
       c->map_num, sizeof(struct icbmap),
       c->map_num * sizeof(struct icbmap));
     */
@@ -324,7 +326,7 @@ static int SetUDFCache(dvd_reader_t *device, UDFCacheType type,
                   | ((uint32_t)data[(p) + 1] << 8)      \
                   | ((uint32_t)data[(p) + 2] << 16)     \
                   | ((uint32_t)data[(p) + 3] << 24))
-/* This is wrong with regard to endianess */
+/* This is wrong with regard to endianness */
 #define GETN(p, n, target) memcpy(target, &data[p], n)
 
 static int Unicodedecode( uint8_t *data, int len, char *target )
@@ -413,14 +415,15 @@ static int UDFPartition( uint8_t *data, uint16_t *Flags, uint16_t *Number,
  * Reads the volume descriptor and checks the parameters.  Returns 0 on OK, 1
  * on error.
  */
-static int UDFLogVolume( uint8_t *data, char *VolumeDescriptor )
+static int UDFLogVolume( uint8_t *data )
 {
-  uint32_t lbsize, MT_L, N_PM;
-  Unicodedecode(&data[84], 128, VolumeDescriptor);
+  uint32_t lbsize;
   lbsize = GETN4(212);  /* should be 2048 */
-  MT_L = GETN4(264);    /* should be 6 */
-  N_PM = GETN4(268);    /* should be 1 */
-  if (lbsize != DVD_VIDEO_LB_LEN) return 1;
+  /* MT_L = GETN4(264);  */  /* should be 6 */
+  /* N_PM = GETN4(268);  */  /* should be 1 */
+  if (lbsize != DVD_VIDEO_LB_LEN)
+    return 1;
+
   return 0;
 }
 
@@ -430,6 +433,7 @@ static int UDFFileEntry( uint8_t *data, uint8_t *FileType,
   uint16_t flags;
   uint32_t L_EA, L_AD;
   unsigned int p;
+  struct AD tmpad;
 
   UDFICB( &data[ 16 ], FileType, &flags );
 
@@ -449,10 +453,12 @@ static int UDFFileEntry( uint8_t *data, uint8_t *FileType,
   while( p < 176 + L_EA + L_AD ) {
     switch( flags & 0x0007 ) {
     case 0:
-      UDFShortAD( &data[ p ], ad, partition );
+      UDFShortAD( &data[ p ], &tmpad, partition );
+      if( ad->Location == 0 ) /* We do not support more than one AD */
+          *ad = tmpad;        /* This was exploited by copy protections adding empty AD */
       p += 8;
       break;
-    case 1:
+    case 1: /* FIXME ? UDF 2.60 2.3.7.1 Only Short Allocation Descriptors shall be used */
       UDFLongAD( &data[ p ], ad );
       p += 16;
       break;
@@ -505,7 +511,7 @@ static int UDFFileIdentifier( uint8_t *data, uint8_t *FileCharacteristics,
  * File: Location of file the ICB is pointing to
  * return 1 on success, 0 on error;
  */
-static int UDFMapICB( dvd_reader_t *device, struct AD ICB, uint8_t *FileType,
+static int UDFMapICB( dvd_reader_t *ctx, struct AD ICB, uint8_t *FileType,
                       struct Partition *partition, struct AD *File )
 {
   uint8_t LogBlock_base[DVD_VIDEO_LB_LEN + 2048];
@@ -513,30 +519,37 @@ static int UDFMapICB( dvd_reader_t *device, struct AD ICB, uint8_t *FileType,
   uint32_t lbnum;
   uint16_t TagID;
   struct icbmap tmpmap;
+  int ret;
 
   lbnum = partition->Start + ICB.Location;
   tmpmap.lbn = lbnum;
-  if(GetUDFCache(device, MapCache, lbnum, &tmpmap)) {
+  if(GetUDFCache(ctx, MapCache, lbnum, &tmpmap)) {
     *FileType = tmpmap.filetype;
     memcpy(File, &tmpmap.file, sizeof(tmpmap.file));
     return 1;
   }
 
   do {
-    if( DVDReadLBUDF( device, lbnum++, 1, LogBlock, 0 ) <= 0 )
+    ret = DVDReadLBUDF( ctx, lbnum++, 1, LogBlock, 0 );
+    if( ret < 0 ) {
+      return ret;
+    }
+    else if( ret == 0 ) {
       TagID = 0;
-    else
+    }
+    else {
       UDFDescriptor( LogBlock, &TagID );
+    }
 
-    if( TagID == 261 ) {
+    if( TagID == FileEntry ) {
       UDFFileEntry( LogBlock, FileType, partition, File );
       memcpy(&tmpmap.file, File, sizeof(tmpmap.file));
       tmpmap.filetype = *FileType;
-      SetUDFCache(device, MapCache, tmpmap.lbn, &tmpmap);
+      SetUDFCache(ctx, MapCache, tmpmap.lbn, &tmpmap);
       return 1;
     };
   } while( ( lbnum <= partition->Start + ICB.Location + ( ICB.Length - 1 )
-             / DVD_VIDEO_LB_LEN ) && ( TagID != 261 ) );
+             / DVD_VIDEO_LB_LEN ) && ( TagID != FileEntry ) );
 
   return 0;
 }
@@ -547,7 +560,7 @@ static int UDFMapICB( dvd_reader_t *device, struct AD ICB, uint8_t *FileType,
  * FileICB: Location of ICB of the found file
  * return 1 on success, 0 on error;
  */
-static int UDFScanDir( dvd_reader_t *device, struct AD Dir, char *FileName,
+static int UDFScanDir( dvd_reader_t *ctx, struct AD Dir, char *FileName,
                        struct Partition *partition, struct AD *FileICB,
                        int cache_file_info)
 {
@@ -561,35 +574,40 @@ static int UDFScanDir( dvd_reader_t *device, struct AD Dir, char *FileName,
   uint8_t *cached_dir_base = NULL, *cached_dir;
   uint32_t dir_lba;
   struct AD tmpICB;
-  int found = 0;
-  int in_cache = 0;
+  int ret;
 
   /* Scan dir for ICB of file */
   lbnum = partition->Start + Dir.Location;
 
-  if(DVDUDFCacheLevel(device, -1) > 0) {
+  if(DVDUDFCacheLevel(ctx, -1) > 0) {
+    int found = 0;
+    int in_cache = 0;
+
     /* caching */
 
-    if(!GetUDFCache(device, LBUDFCache, lbnum, &cached_dir)) {
+    if(!GetUDFCache(ctx, LBUDFCache, lbnum, &cached_dir)) {
       dir_lba = (Dir.Length + DVD_VIDEO_LB_LEN) / DVD_VIDEO_LB_LEN;
       if((cached_dir_base = malloc(dir_lba * DVD_VIDEO_LB_LEN + 2048)) == NULL)
         return 0;
       cached_dir = (uint8_t *)(((uintptr_t)cached_dir_base & ~((uintptr_t)2047)) + 2048);
-      if( DVDReadLBUDF( device, lbnum, dir_lba, cached_dir, 0) <= 0 ) {
+      ret = DVDReadLBUDF( ctx, lbnum, dir_lba, cached_dir, 0);
+      if( ret <= 0 ) {
         free(cached_dir_base);
         cached_dir_base = NULL;
         cached_dir = NULL;
+        if( ret < 0 )
+            return ret;
       }
       /*
       if(cached_dir) {
-        fprintf(stderr, "malloc dir: %d\n",  dir_lba * DVD_VIDEO_LB_LEN);
+        Log3(ctx, "malloc dir: %d",  dir_lba * DVD_VIDEO_LB_LEN);
       }
       */
       {
         uint8_t *data[2];
         data[0] = cached_dir_base;
         data[1] = cached_dir;
-        SetUDFCache(device, LBUDFCache, lbnum, data);
+        SetUDFCache(ctx, LBUDFCache, lbnum, data);
       }
     } else
       in_cache = 1;
@@ -602,8 +620,9 @@ static int UDFScanDir( dvd_reader_t *device, struct AD Dir, char *FileName,
     p = 0;
 
     while( p < Dir.Length ) {
+
       UDFDescriptor( &cached_dir[ p ], &TagID );
-      if( TagID == 257 ) {
+      if( TagID == FileIdentifierDescriptor ) {
         p += UDFFileIdentifier( &cached_dir[ p ], &filechar,
                                 filename, &tmpICB );
         if(cache_file_info && !in_cache) {
@@ -614,7 +633,9 @@ static int UDFScanDir( dvd_reader_t *device, struct AD Dir, char *FileName,
             memcpy(FileICB, &tmpICB, sizeof(tmpICB));
             found = 1;
           }
-          UDFMapICB(device, tmpICB, &tmpFiletype, partition, &tmpFile);
+          if(!UDFMapICB(ctx, tmpICB, &tmpFiletype, partition, &tmpFile))
+            return 0;
+
         } else {
           if( !strcasecmp( FileName, filename ) ) {
             memcpy(FileICB, &tmpICB, sizeof(tmpICB));
@@ -632,7 +653,7 @@ static int UDFScanDir( dvd_reader_t *device, struct AD Dir, char *FileName,
     return 0;
   }
 
-  if( DVDReadLBUDF( device, lbnum, 2, directory, 0 ) <= 0 )
+  if( DVDReadLBUDF( ctx, lbnum, 2, directory, 0 ) <= 0 )
     return 0;
 
   p = 0;
@@ -641,12 +662,12 @@ static int UDFScanDir( dvd_reader_t *device, struct AD Dir, char *FileName,
       ++lbnum;
       p -= DVD_VIDEO_LB_LEN;
       Dir.Length -= DVD_VIDEO_LB_LEN;
-      if( DVDReadLBUDF( device, lbnum, 2, directory, 0 ) <= 0 ) {
+      if( DVDReadLBUDF( ctx, lbnum, 2, directory, 0 ) <= 0 ) {
         return 0;
       }
     }
     UDFDescriptor( &directory[ p ], &TagID );
-    if( TagID == 257 ) {
+    if( TagID == FileIdentifierDescriptor ) {
       p += UDFFileIdentifier( &directory[ p ], &filechar,
                               filename, FileICB );
       if( !strcasecmp( FileName, filename ) ) {
@@ -660,7 +681,7 @@ static int UDFScanDir( dvd_reader_t *device, struct AD Dir, char *FileName,
 }
 
 
-static int UDFGetAVDP( dvd_reader_t *device,
+static int UDFGetAVDP( dvd_reader_t *ctx,
                        struct avdp_t *avdp)
 {
   uint8_t Anchor_base[ DVD_VIDEO_LB_LEN + 2048 ];
@@ -670,8 +691,9 @@ static int UDFGetAVDP( dvd_reader_t *device,
   uint32_t lastsector;
   int terminate;
   struct avdp_t;
+  int ret;
 
-  if(GetUDFCache(device, AVDPCache, 0, avdp))
+  if(GetUDFCache(ctx, AVDPCache, 0, avdp))
     return 1;
 
   /* Find Anchor */
@@ -680,12 +702,17 @@ static int UDFGetAVDP( dvd_reader_t *device,
   terminate = 0;
 
   for(;;) {
-    if( DVDReadLBUDF( device, lbnum, 1, Anchor, 0 ) > 0 ) {
-      UDFDescriptor( Anchor, &TagID );
-    } else {
+    ret = DVDReadLBUDF( ctx, lbnum, 1, Anchor, 0 );
+    if( ret < 0 ) {
+      return ret;
+    }
+    else if( ret == 0 ) {
       TagID = 0;
     }
-    if (TagID != 2) {
+    else {
+      UDFDescriptor( Anchor, &TagID );
+    }
+    if (TagID != AnchorVolumeDescriptorPointer) {
       /* Not an anchor */
       if( terminate ) return 0; /* Final try failed */
 
@@ -718,7 +745,7 @@ static int UDFGetAVDP( dvd_reader_t *device,
   avdp->rvds.location = MVDS_location;
   avdp->rvds.length = MVDS_length;
 
-  SetUDFCache(device, AVDPCache, 0, avdp);
+  SetUDFCache(ctx, AVDPCache, 0, avdp);
 
   return 1;
 }
@@ -728,17 +755,17 @@ static int UDFGetAVDP( dvd_reader_t *device,
  *   partnum: Number of the partition, starting at 0.
  *   part: structure to fill with the partition information
  */
-static int UDFFindPartition( dvd_reader_t *device, int partnum,
+static int UDFFindPartition( dvd_reader_t *ctx, int partnum,
                              struct Partition *part )
 {
   uint8_t LogBlock_base[ DVD_VIDEO_LB_LEN + 2048 ];
   uint8_t *LogBlock = (uint8_t *)(((uintptr_t)LogBlock_base & ~((uintptr_t)2047)) + 2048);
   uint32_t lbnum, MVDS_location, MVDS_length;
   uint16_t TagID;
-  int i, volvalid;
+  int i, volvalid, ret;
   struct avdp_t avdp;
 
-  if(!UDFGetAVDP(device, &avdp))
+  if(!UDFGetAVDP(ctx, &avdp))
     return 0;
 
   /* Main volume descriptor */
@@ -747,33 +774,38 @@ static int UDFFindPartition( dvd_reader_t *device, int partnum,
 
   part->valid = 0;
   volvalid = 0;
-  part->VolumeDesc[ 0 ] = '\0';
   i = 1;
   do {
     /* Find Volume Descriptor */
     lbnum = MVDS_location;
     do {
 
-      if( DVDReadLBUDF( device, lbnum++, 1, LogBlock, 0 ) <= 0 )
+      ret = DVDReadLBUDF( ctx, lbnum++, 1, LogBlock, 0 );
+      if( ret < 0 ) {
+        return ret;
+      }
+      else if( ret == 0 ) {
         TagID = 0;
-      else
+      }
+      else {
         UDFDescriptor( LogBlock, &TagID );
+      }
 
-      if( ( TagID == 5 ) && ( !part->valid ) ) {
+      if( ( TagID == PartitionDescriptor ) && ( !part->valid ) ) {
         /* Partition Descriptor */
         UDFPartition( LogBlock, &part->Flags, &part->Number,
                       part->Contents, &part->Start, &part->Length );
         part->valid = ( partnum == part->Number );
-      } else if( ( TagID == 6 ) && ( !volvalid ) ) {
+      } else if( ( TagID == LogicalVolumeDescriptor ) && ( !volvalid ) ) {
         /* Logical Volume Descriptor */
-        if( UDFLogVolume( LogBlock, part->VolumeDesc ) ) {
+        if( UDFLogVolume( LogBlock ) ) {
           /* TODO: sector size wrong! */
         } else
           volvalid = 1;
       }
 
     } while( ( lbnum <= MVDS_location + ( MVDS_length - 1 )
-               / DVD_VIDEO_LB_LEN ) && ( TagID != 8 )
+               / DVD_VIDEO_LB_LEN ) && ( TagID != TerminatingDescriptor )
              && ( ( !part->valid ) || ( !volvalid ) ) );
 
     if( ( !part->valid) || ( !volvalid ) ) {
@@ -787,7 +819,7 @@ static int UDFFindPartition( dvd_reader_t *device, int partnum,
   return part->valid;
 }
 
-uint32_t UDFFindFile( dvd_reader_t *device, char *filename,
+uint32_t UDFFindFile( dvd_reader_t *ctx, const char *filename,
                       uint32_t *filesize )
 {
   uint8_t LogBlock_base[ DVD_VIDEO_LB_LEN + 2048 ];
@@ -797,59 +829,65 @@ uint32_t UDFFindFile( dvd_reader_t *device, char *filename,
   struct Partition partition;
   struct AD RootICB, File, ICB;
   char tokenline[ MAX_UDF_FILE_NAME_LEN ];
-  char *token;
   uint8_t filetype;
+  int ret;
 
   *filesize = 0;
   tokenline[0] = '\0';
   strncat(tokenline, filename, MAX_UDF_FILE_NAME_LEN - 1);
   memset(&ICB, 0, sizeof(ICB));
 
-  if(!(GetUDFCache(device, PartitionCache, 0, &partition) &&
-       GetUDFCache(device, RootICBCache, 0, &RootICB))) {
+  if(!(GetUDFCache(ctx, PartitionCache, 0, &partition) &&
+       GetUDFCache(ctx, RootICBCache, 0, &RootICB))) {
     /* Find partition, 0 is the standard location for DVD Video.*/
-    if( !UDFFindPartition( device, 0, &partition ) ) return 0;
-    SetUDFCache(device, PartitionCache, 0, &partition);
+    if( !UDFFindPartition( ctx, 0, &partition ) ) return 0;
+    SetUDFCache(ctx, PartitionCache, 0, &partition);
 
     /* Find root dir ICB */
     lbnum = partition.Start;
     do {
-      if( DVDReadLBUDF( device, lbnum++, 1, LogBlock, 0 ) <= 0 )
+      ret = DVDReadLBUDF( ctx, lbnum++, 1, LogBlock, 0 );
+      if( ret < 0 ) {
+        return ret;
+      }
+      else if( ret == 0 ) {
         TagID = 0;
-      else
+      }
+      else {
         UDFDescriptor( LogBlock, &TagID );
+      }
 
       /* File Set Descriptor */
-      if( TagID == 256 )  /* File Set Descriptor */
+      if( TagID == FileSetDescriptor )  /* File Set Descriptor */
         UDFLongAD( &LogBlock[ 400 ], &RootICB );
     } while( ( lbnum < partition.Start + partition.Length )
-             && ( TagID != 8 ) && ( TagID != 256 ) );
+             && ( TagID != TerminatingDescriptor ) && ( TagID != FileSetDescriptor) );
 
     /* Sanity checks. */
-    if( TagID != 256 )
+    if( TagID != FileSetDescriptor )
       return 0;
     if( RootICB.Partition != 0 )
       return 0;
-    SetUDFCache(device, RootICBCache, 0, &RootICB);
+    SetUDFCache(ctx, RootICBCache, 0, &RootICB);
   }
 
   /* Find root dir */
-  if( !UDFMapICB( device, RootICB, &filetype, &partition, &File ) )
+  if( !UDFMapICB( ctx, RootICB, &filetype, &partition, &File ) )
     return 0;
   if( filetype != 4 )
     return 0;  /* Root dir should be dir */
   {
     int cache_file_info = 0;
     /* Tokenize filepath */
-    token = strtok(tokenline, "/");
+    char *token = strtok(tokenline, "/");
 
     while( token != NULL ) {
-      if( !UDFScanDir( device, File, token, &partition, &ICB,
+      if( !UDFScanDir( ctx, File, token, &partition, &ICB,
                        cache_file_info))
         return 0;
-      if( !UDFMapICB( device, ICB, &filetype, &partition, &File ) )
+      if( !UDFMapICB( ctx, ICB, &filetype, &partition, &File ) )
         return 0;
-      if(!strcmp(token, "VIDEO_TS"))
+      if(!strcmp(token, DVD_TYPE_STRING( ctx->dvd_type ) ))
         cache_file_info = 1;
       token = strtok( NULL, "/" );
     }
@@ -874,23 +912,19 @@ uint32_t UDFFindFile( dvd_reader_t *device, char *filename,
  * id, tagid of descriptor
  * bufsize, size of BlockBuf (must be >= DVD_VIDEO_LB_LEN).
  */
-static int UDFGetDescriptor( dvd_reader_t *device, int id,
+static int UDFGetDescriptor( dvd_reader_t *ctx, int id,
                              uint8_t *descriptor, int bufsize)
 {
   uint32_t lbnum, MVDS_location, MVDS_length;
   struct avdp_t avdp;
   uint16_t TagID;
-  uint32_t lastsector;
-  int i, terminate;
-  int desc_found = 0;
+  int i, desc_found = 0, ret;
   /* Find Anchor */
-  lastsector = 0;
   lbnum = 256;   /* Try #1, prime anchor */
-  terminate = 0;
   if(bufsize < DVD_VIDEO_LB_LEN)
     return 0;
 
-  if(!UDFGetAVDP(device, &avdp))
+  if(!UDFGetAVDP(ctx, &avdp))
     return 0;
 
   /* Main volume descriptor */
@@ -902,15 +936,21 @@ static int UDFGetDescriptor( dvd_reader_t *device, int id,
     /* Find  Descriptor */
     lbnum = MVDS_location;
     do {
-      if( DVDReadLBUDF( device, lbnum++, 1, descriptor, 0 ) <= 0 )
+      ret = DVDReadLBUDF( ctx, lbnum++, 1, descriptor, 0 );
+      if( ret < 0 ) {
+        return ret;
+      }
+      else if( ret == 0 ) {
         TagID = 0;
-      else
+      }
+      else {
         UDFDescriptor( descriptor, &TagID );
+      }
       if( (TagID == id) && ( !desc_found ) )
         /* Descriptor */
         desc_found = 1;
     } while( ( lbnum <= MVDS_location + ( MVDS_length - 1 )
-               / DVD_VIDEO_LB_LEN ) && ( TagID != 8 )
+               / DVD_VIDEO_LB_LEN ) && ( TagID != TerminatingDescriptor )
              && ( !desc_found) );
 
     if( !desc_found ) {
@@ -924,19 +964,19 @@ static int UDFGetDescriptor( dvd_reader_t *device, int id,
 }
 
 
-static int UDFGetPVD(dvd_reader_t *device, struct pvd_t *pvd)
+static int UDFGetPVD(dvd_reader_t *ctx, struct pvd_t *pvd)
 {
   uint8_t pvd_buf_base[DVD_VIDEO_LB_LEN + 2048];
   uint8_t *pvd_buf = (uint8_t *)(((uintptr_t)pvd_buf_base & ~((uintptr_t)2047)) + 2048);
-  if(GetUDFCache(device, PVDCache, 0, pvd))
+  if(GetUDFCache(ctx, PVDCache, 0, pvd))
     return 1;
 
-  if(!UDFGetDescriptor( device, 1, pvd_buf, DVD_VIDEO_LB_LEN))
+  if(!UDFGetDescriptor( ctx, 1, pvd_buf, DVD_VIDEO_LB_LEN))
     return 0;
 
   memcpy(pvd->VolumeIdentifier, &pvd_buf[24], 32);
   memcpy(pvd->VolumeSetIdentifier, &pvd_buf[72], 128);
-  SetUDFCache(device, PVDCache, 0, pvd);
+  SetUDFCache(ctx, PVDCache, 0, pvd);
   return 1;
 }
 
@@ -946,14 +986,14 @@ static int UDFGetPVD(dvd_reader_t *device, struct pvd_t *pvd)
  * volid_size, size of the buffer volid points to
  * returns the size of buffer needed for all data
  */
-int UDFGetVolumeIdentifier(dvd_reader_t *device, char *volid,
+int UDFGetVolumeIdentifier(dvd_reader_t *ctx, char *volid,
                            unsigned int volid_size)
 {
   struct pvd_t pvd;
   unsigned int volid_len;
 
   /* get primary volume descriptor */
-  if(!UDFGetPVD(device, &pvd))
+  if(!UDFGetPVD(ctx, &pvd))
     return 0;
 
   volid_len = pvd.VolumeIdentifier[31];
@@ -976,13 +1016,13 @@ int UDFGetVolumeIdentifier(dvd_reader_t *device, char *volid,
  * returns the size of the available volsetid information (128)
  * or 0 on error
  */
-int UDFGetVolumeSetIdentifier(dvd_reader_t *device, uint8_t *volsetid,
+int UDFGetVolumeSetIdentifier(dvd_reader_t *ctx, uint8_t *volsetid,
                               unsigned int volsetid_size)
 {
   struct pvd_t pvd;
 
   /* get primary volume descriptor */
-  if(!UDFGetPVD(device, &pvd))
+  if(!UDFGetPVD(ctx, &pvd))
     return 0;
 
 
