@@ -23,22 +23,20 @@
 #define LIBDVDREAD_IFO_TYPES_H
 
 #include <inttypes.h>
-#include "dvdread/dvd_reader.h"
 
+#include <dvdread/dvd_reader.h>
 
 #undef ATTRIBUTE_PACKED
-#undef PRAGMA_PACK_BEGIN
-#undef PRAGMA_PACK_END
 
 #if defined(__GNUC__)
-#if __GNUC__ > 2 || (__GNUC__ == 2 && __GNUC_MINOR__ >= 95)
-#if __GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4)
-#define ATTRIBUTE_PACKED __attribute__ ((packed,gcc_struct))
-#else
-#define ATTRIBUTE_PACKED __attribute__ ((packed))
-#endif
-#define PRAGMA_PACK 0
-#endif
+# if __GNUC__ > 2 || (__GNUC__ == 2 && __GNUC_MINOR__ >= 95)
+#  if (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4)) && !defined(__clang__)
+#   define ATTRIBUTE_PACKED __attribute__ ((packed,gcc_struct))
+#  else
+#   define ATTRIBUTE_PACKED __attribute__ ((packed))
+#  endif
+#  define PRAGMA_PACK 0
+# endif
 #endif
 
 #if !defined(ATTRIBUTE_PACKED)
@@ -67,6 +65,7 @@ typedef struct {
   uint8_t second;
   uint8_t frame_u; /* The two high bits are the frame rate. */
 } ATTRIBUTE_PACKED dvd_time_t;
+#define DVD_TIME_SIZE 4U
 
 /**
  * Type to store per-command data.
@@ -95,6 +94,7 @@ typedef struct {
   unsigned char letterboxed          : 1;
   unsigned char film_mode            : 1;
 } ATTRIBUTE_PACKED video_attr_t;
+#define VIDEO_ATTR_SIZE  2U
 
 /**
  * Audio Attributes.
@@ -128,37 +128,39 @@ typedef struct {
     } surround;
   } ATTRIBUTE_PACKED app_info;
 } ATTRIBUTE_PACKED audio_attr_t;
+#define AUDIO_ATTR_SIZE  8U
 
 
 /**
  * MultiChannel Extension
  */
 typedef struct {
-  unsigned int zero1      : 7;
-  unsigned int ach0_gme   : 1;
+  unsigned char zero1      : 7;
+  unsigned char ach0_gme   : 1;
 
-  unsigned int zero2      : 7;
-  unsigned int ach1_gme   : 1;
+  unsigned char zero2      : 7;
+  unsigned char ach1_gme   : 1;
 
-  unsigned int zero3      : 4;
-  unsigned int ach2_gv1e  : 1;
-  unsigned int ach2_gv2e  : 1;
-  unsigned int ach2_gm1e  : 1;
-  unsigned int ach2_gm2e  : 1;
+  unsigned char zero3      : 4;
+  unsigned char ach2_gv1e  : 1;
+  unsigned char ach2_gv2e  : 1;
+  unsigned char ach2_gm1e  : 1;
+  unsigned char ach2_gm2e  : 1;
 
-  unsigned int zero4      : 4;
-  unsigned int ach3_gv1e  : 1;
-  unsigned int ach3_gv2e  : 1;
-  unsigned int ach3_gmAe  : 1;
-  unsigned int ach3_se2e  : 1;
+  unsigned char zero4      : 4;
+  unsigned char ach3_gv1e  : 1;
+  unsigned char ach3_gv2e  : 1;
+  unsigned char ach3_gmAe  : 1;
+  unsigned char ach3_se2e  : 1;
 
-  unsigned int zero5      : 4;
-  unsigned int ach4_gv1e  : 1;
-  unsigned int ach4_gv2e  : 1;
-  unsigned int ach4_gmBe  : 1;
-  unsigned int ach4_seBe  : 1;
+  unsigned char zero5      : 4;
+  unsigned char ach4_gv1e  : 1;
+  unsigned char ach4_gv2e  : 1;
+  unsigned char ach4_gmBe  : 1;
+  unsigned char ach4_seBe  : 1;
   uint8_t zero6[19];
 } ATTRIBUTE_PACKED multichannel_ext_t;
+#define MULTICHANNEL_EXT_SIZE  24U
 
 
 /**
@@ -183,6 +185,7 @@ typedef struct {
   uint8_t  lang_extension;
   uint8_t  code_extension;
 } ATTRIBUTE_PACKED subp_attr_t;
+#define SUBP_ATTR_SIZE  6U
 
 
 
@@ -193,7 +196,7 @@ typedef struct {
   uint16_t nr_of_pre;
   uint16_t nr_of_post;
   uint16_t nr_of_cell;
-  uint16_t zero_1;
+  uint16_t last_byte;
   vm_cmd_t *pre_cmds;
   vm_cmd_t *post_cmds;
   vm_cmd_t *cell_cmds;
@@ -209,16 +212,16 @@ typedef uint8_t pgc_program_map_t;
  * Cell Playback Information.
  */
 typedef struct {
-  unsigned int block_mode       : 2;
-  unsigned int block_type       : 2;
-  unsigned int seamless_play    : 1;
-  unsigned int interleaved      : 1;
-  unsigned int stc_discontinuity: 1;
-  unsigned int seamless_angle   : 1;
-
-  unsigned int playback_mode    : 1;  /**< When set, enter StillMode after each VOBU */
-  unsigned int restricted       : 1;  /**< ?? drop out of fastforward? */
-  unsigned int unknown2         : 6;
+  unsigned char block_mode       : 2;
+  unsigned char block_type       : 2;
+  unsigned char seamless_play    : 1;
+  unsigned char interleaved      : 1;
+  unsigned char stc_discontinuity: 1;
+  unsigned char seamless_angle   : 1;
+  unsigned char zero_1           : 1;
+  unsigned char playback_mode    : 1;  /**< When set, enter StillMode after each VOBU */
+  unsigned char restricted       : 1;  /**< ?? drop out of fastforward? */
+  unsigned char cell_type        : 5;  /** for karaoke, reserved otherwise */
   uint8_t still_time;
   uint8_t cell_cmd_nr;
   dvd_time_t playback_time;
@@ -227,6 +230,7 @@ typedef struct {
   uint32_t last_vobu_start_sector;
   uint32_t last_sector;
 } ATTRIBUTE_PACKED cell_playback_t;
+#define CELL_PLAYBACK_SIZE  24U
 
 #define BLOCK_TYPE_NONE         0x0
 #define BLOCK_TYPE_ANGLE_BLOCK  0x1
@@ -244,41 +248,43 @@ typedef struct {
   uint8_t  zero_1;
   uint8_t  cell_nr;
 } ATTRIBUTE_PACKED cell_position_t;
+#define CELL_POSITION_SIZE 4U
 
 /**
  * User Operations.
  */
 typedef struct {
-  unsigned int zero                           : 7; /* 25-31 */
-  unsigned int video_pres_mode_change         : 1; /* 24 */
+  unsigned char zero                           : 7; /* 25-31 */
+  unsigned char video_pres_mode_change         : 1; /* 24 */
 
-  unsigned int karaoke_audio_pres_mode_change : 1; /* 23 */
-  unsigned int angle_change                   : 1;
-  unsigned int subpic_stream_change           : 1;
-  unsigned int audio_stream_change            : 1;
-  unsigned int pause_on                       : 1;
-  unsigned int still_off                      : 1;
-  unsigned int button_select_or_activate      : 1;
-  unsigned int resume                         : 1; /* 16 */
+  unsigned char karaoke_audio_pres_mode_change : 1; /* 23 */
+  unsigned char angle_change                   : 1;
+  unsigned char subpic_stream_change           : 1;
+  unsigned char audio_stream_change            : 1;
+  unsigned char pause_on                       : 1;
+  unsigned char still_off                      : 1;
+  unsigned char button_select_or_activate      : 1;
+  unsigned char resume                         : 1; /* 16 */
 
-  unsigned int chapter_menu_call              : 1; /* 15 */
-  unsigned int angle_menu_call                : 1;
-  unsigned int audio_menu_call                : 1;
-  unsigned int subpic_menu_call               : 1;
-  unsigned int root_menu_call                 : 1;
-  unsigned int title_menu_call                : 1;
-  unsigned int backward_scan                  : 1;
-  unsigned int forward_scan                   : 1; /* 8 */
+  unsigned char chapter_menu_call              : 1; /* 15 */
+  unsigned char angle_menu_call                : 1;
+  unsigned char audio_menu_call                : 1;
+  unsigned char subpic_menu_call               : 1;
+  unsigned char root_menu_call                 : 1;
+  unsigned char title_menu_call                : 1;
+  unsigned char backward_scan                  : 1;
+  unsigned char forward_scan                   : 1; /* 8 */
 
-  unsigned int next_pg_search                 : 1; /* 7 */
-  unsigned int prev_or_top_pg_search          : 1;
-  unsigned int time_or_chapter_search         : 1;
-  unsigned int go_up                          : 1;
-  unsigned int stop                           : 1;
-  unsigned int title_play                     : 1;
-  unsigned int chapter_search_or_play         : 1;
-  unsigned int title_or_time_play             : 1; /* 0 */
+  unsigned char next_pg_search                 : 1; /* 7 */
+  unsigned char prev_or_top_pg_search          : 1;
+  unsigned char time_or_chapter_search         : 1;
+  unsigned char go_up                          : 1;
+  unsigned char stop                           : 1;
+  unsigned char title_play                     : 1;
+  unsigned char chapter_search_or_play         : 1;
+  unsigned char title_or_time_play             : 1; /* 0 */
 } ATTRIBUTE_PACKED user_ops_t;
+#define USER_OPS_SIZE  4U
 
 /**
  * Program Chain Information.
@@ -294,8 +300,8 @@ typedef struct {
   uint16_t next_pgc_nr;
   uint16_t prev_pgc_nr;
   uint16_t goup_pgc_nr;
-  uint8_t  still_time;
   uint8_t  pg_playback_mode;
+  uint8_t  still_time;
   uint32_t palette[16]; /* New type struct {zero_1, Y, Cr, Cb} ? */
   uint16_t command_tbl_offset;
   uint16_t program_map_offset;
@@ -314,9 +320,9 @@ typedef struct {
  */
 typedef struct {
   uint8_t  entry_id;
-  unsigned int block_mode : 2;
-  unsigned int block_type : 2;
-  unsigned int unknown1   : 4;
+  unsigned char block_mode : 2;
+  unsigned char block_type : 2;
+  unsigned char zero_1   : 4;
   uint16_t ptl_id_mask;
   uint32_t pgc_start_byte;
   pgc_t *pgc;
@@ -368,6 +374,7 @@ typedef struct {
   uint32_t start_sector;
   uint32_t last_sector;
 } ATTRIBUTE_PACKED cell_adr_t;
+#define CELL_ADDR_SIZE  12U
 
 /**
  * Cell Address Table.
@@ -440,17 +447,172 @@ typedef struct {
   subp_attr_t  vmgm_subp_attr;
   subp_attr_t  zero_10[27];  /* XXX: how much 'padding' here? */
 } ATTRIBUTE_PACKED vmgi_mat_t;
+#define VMGI_MAT_SIZE 510U
+
+/* Downmix coefficients can be used to reduce 5.1 channels to stereo in DVD-Audio Discs */
+/** Downmix equations
+ * Left_out  = Lf_left  * Lf
+ *           + Rf_left  * Rf
+ *           + C_left   * C
+ *           + LFE_left * LFE
+ *           + Ls_left  * Ls
+ *           + Rs_left  * Rs;
+ * 
+ * Right_out = Lf_right  * Lf
+ *           + Rf_right  * Rf
+ *           + C_right   * C
+ *           + LFE_right * LFE
+ *           + Ls_right  * Ls
+ *           + Rs_right  * Rs;
+ * 
+ * Where:
+ *   - Lf, Rf, C, LFE, Ls, Rs are the 5.1 input channels
+ *   - Left_out, Right_out are the stereo output channels
+ *   - Each coefficient (e.g. Lf_left, C_right) is an 8-bit gain factor
+ */
 
 typedef struct {
-  unsigned int zero_1                    : 1;
-  unsigned int multi_or_random_pgc_title : 1; /* 0: one sequential pgc title */
-  unsigned int jlc_exists_in_cell_cmd    : 1;
-  unsigned int jlc_exists_in_prepost_cmd : 1;
-  unsigned int jlc_exists_in_button_cmd  : 1;
-  unsigned int jlc_exists_in_tt_dom      : 1;
-  unsigned int chapter_search_or_play    : 1; /* UOP 1 */
-  unsigned int title_or_time_play        : 1; /* UOP 0 */
+  /* it seems each entry is started and ended with padding */
+  uint16_t zero_1;
+  /* each coefficient corresponds to stereo side for a channel in 5.1 */
+  uint8_t Lf_left;
+  uint8_t Lf_right;
+  uint8_t Rf_left;
+  uint8_t Rf_right;
+  uint8_t C_left;
+  uint8_t C_right;
+  uint8_t LFE_left;
+  uint8_t LFE_right;
+  uint8_t Ls_left;
+  uint8_t Ls_right;
+  uint8_t Rs_left;
+  uint8_t Rs_right;
+
+  uint16_t zero_2;
+} ATTRIBUTE_PACKED downmix_coeff_t;
+#define DOWNMIX_COEFF_SIZE 16U
+
+
+/**
+ * SAMG
+ *
+ * The following structures relate to the Simple Audio Manager, exclusive to DVD-Audio discs
+ */
+
+/**
+ * This is one of two of the DVD-A content managers, for simple audio dvd only players
+ */
+
+typedef struct {
+  uint16_t zero_1;
+  uint8_t  group_num;
+  uint8_t  chapter_num; /* chapter number within group*/
+  uint32_t timestamp_pts; /* this is MPEG time, Not DVD time */
+  uint32_t chapter_len;
+  uint32_t zero_2;
+  uint8_t  record_code;
+  uint8_t  bit_depth;
+  uint8_t  sampling_rate;
+  uint8_t  nr_channels; 
+  /* some DVD's made with authoring software keep downmix coefficients here */
+  /* since I do not have samples of commercial discs that do this, I will not include it */
+  uint8_t  zero_3[20];
+  uint32_t start_sector_1; /*aob start sector*/
+  uint32_t start_sector_2; /*aob start sector is repeated again */
+  uint32_t end_sector;
+} ATTRIBUTE_PACKED samg_chapter_t;
+#define SAMG_CHAPTER_SIZE 52U
+
+typedef struct {
+  char           samg_identifier[12];
+  uint16_t       nr_chapters;
+  uint16_t       specification_version;
+  samg_chapter_t *samg_chapters;
+} ATTRIBUTE_PACKED samg_mat_t;
+#define SAMG_MAT_SIZE 16U
+
+/**
+ * AMGI
+ *
+ * The following structures relate to the Audio Manager, exclusive to DVD-Audio discs
+ */
+
+/**
+ * Audio Manager Information Management Table.
+ */
+typedef struct {
+  char     amg_identifier[12];
+  uint32_t amg_start_sector;
+  uint8_t  zero_1[12];
+  uint32_t amgi_last_sector;
+  uint16_t specification_version;
+  uint8_t  zero_2[4];
+  uint16_t amg_nr_of_volumes;
+  uint16_t amg_this_volume_nr;
+  uint8_t  disc_side;
+  uint8_t  zero_3[4];
+  uint8_t  autoplay;
+  uint32_t audio_sv_ifo_relative_p;
+  uint16_t unknown_1; /* some discs have a value here, undocumented in DVD-Audio specs */
+  uint8_t  zero_4[8];
+  uint8_t  vmg_nr_of_title_sets;  /* Number of video titlesets in audio zone. */
+  uint8_t  amg_nr_of_title_sets;  /* Number of audio titlesets in audio zone. */
+  uint8_t  unknown_2[32]; /* may be set to zeros */
+  uint8_t  unknown_3[8]; /* may be set to zeros */
+  uint8_t  zero_5[24];  
+  uint32_t amg_end_byte_address;
+  uint8_t  unknown_4[4]; /* may be set to zeros */
+  uint8_t  zero_6[56];  
+  uint16_t menu_prescence_1; /* may be set to zero, or some other value, optional field*/
+  uint8_t  unknown_5[4];  
+  uint16_t unknown_6; /* should be 0x01 */
+  uint8_t  zero_7[2];  
+  uint16_t amg_nr_of_zones; /* may be set to 0x02*/
+  uint8_t  zero_8[2];  
+  uint16_t menu_prescence_2; /* may be set to 0x03*/
+  uint8_t  zero_9[48];  
+  uint8_t  last_sector_audio_sys_space;  
+  uint8_t  zero_10[79];  
+  uint8_t  menu_prescence_3; /* will be set to 0x01*/
+    /* XXX lots of padding after this to complete the sector*/
+} ATTRIBUTE_PACKED amgi_mat_t;
+#define AMGI_MAT_SIZE 337U
+
+/* Sector 2 may have video tracks, sector 3 will not. If there are no video tracks the tables will be the same*/
+
+/*this struct repeats for every audio or video track*/
+typedef struct {
+  uint8_t  type_and_rank; /*first nibble is type, second rank, from 1-9*/
+  uint8_t  nr_chapters_in_title;
+  uint8_t  nr_visible_chapters_in_vts_title; /* this will be zeros in an audio record */
+  uint8_t  zero_1;
+  uint32_t len_audio_zone_pts; /* this is MPEG time, Not DVD time */
+
+  uint8_t  group_property; /* in a video track, this is video titleset number, in audio its rank of group */
+  uint8_t  title_property; /* in video track this is title number , in audio track this is rank of title*/
+  uint32_t ts_pointer_relative_sector; /* for ats or vts*/
+} ATTRIBUTE_PACKED track_info_t;
+#define TRACK_INFO_SIZE 14U
+
+typedef struct {
+  uint16_t     nr_of_titles;
+  uint16_t     last_byte_in_table;
+  track_info_t *tracks_info;
+} ATTRIBUTE_PACKED tracks_info_table_t;
+#define TRACKS_INFO_TABLE_SIZE 4U
+
+
+typedef struct {
+  unsigned char zero_1                    : 1;
+  unsigned char multi_or_random_pgc_title : 1; /* 0: one sequential pgc title */
+  unsigned char jlc_exists_in_cell_cmd    : 1;
+  unsigned char jlc_exists_in_prepost_cmd : 1;
+  unsigned char jlc_exists_in_button_cmd  : 1;
+  unsigned char jlc_exists_in_tt_dom      : 1;
+  unsigned char chapter_search_or_play    : 1; /* UOP 1 */
+  unsigned char title_or_time_play        : 1; /* UOP 0 */
 } ATTRIBUTE_PACKED playback_type_t;
+#define PLAYBACK_TYPE_SIZE 1U
 
 /**
  * Title Information.
@@ -464,6 +626,7 @@ typedef struct {
   uint8_t  vts_ttn;
   uint32_t title_set_sector;
 } ATTRIBUTE_PACKED title_info_t;
+#define TITLE_INFO_SIZE 12U
 
 /**
  * PartOfTitle Search Pointer Table.
@@ -565,20 +728,22 @@ typedef struct {
   uint8_t unknown1;
   uint8_t unknown2;
   uint8_t unknown3;
-  uint8_t unknown4; /* ?? allways 0x30 language?, text format? */
+  uint8_t unknown4; /* ?? always 0x30 language?, text format? */
   uint8_t unknown5;
   uint16_t offset; /* from first */
 
   char text[12]; /* ended by 0x09 */
 #endif
 } ATTRIBUTE_PACKED txtdt_t;
+#define TXTDT_SIZE 204U
 
 /**
  * Text Data Language Unit. (Incomplete)
  */
 typedef struct {
   uint16_t lang_code;
-  uint16_t unknown;      /* 0x0001, title 1? disc 1? side 1? */
+  uint8_t  zero_1;
+  uint8_t char_set;      /* 0x00 reserved Unicode, 0x01 ISO 646, 0x10 JIS Roman & JIS Kanji, 0x11 ISO 8859-1, 0x12 Shift JIS Kanji */
   uint32_t txtdt_start_byte;  /* prt, rel start of vmg_txtdt_mgi  */
   txtdt_t  *txtdt;
 } ATTRIBUTE_PACKED txtdt_lu_t;
@@ -588,8 +753,9 @@ typedef struct {
  * Text Data Manager Information. (Incomplete)
  */
 typedef struct {
-  char disc_name[14];            /* how many bytes?? */
-  uint16_t nr_of_language_units; /* 32bit??          */
+  char disc_name[12];
+  uint16_t unknown1;
+  uint16_t nr_of_language_units;
   uint32_t last_byte;
   txtdt_lu_t *lu;
 } ATTRIBUTE_PACKED txtdt_mgi_t;
@@ -658,6 +824,118 @@ typedef struct {
   multichannel_ext_t vts_mu_audio_attr[8];
   /* XXX: how much 'padding' here, if any? */
 } ATTRIBUTE_PACKED vtsi_mat_t;
+#define VTSI_MAT_SIZE 984U
+
+
+typedef struct {
+  /* 0x0000 for lpcm or 0x0100 for mlp, DTS seems to be 0x0500 and has different fields*/
+  uint8_t encoding;
+  uint8_t unknown1;
+  /* for stereo 0f 16 bit, 1f 20 bit, 2f 24 bit*/
+  /* f is a placeholder for the nibbles that is used for channel groups*/
+  /* otherwise if its 5.1 channels each nibble represents a channel group (G1, G2),*/
+  uint8_t  bitrate; 
+  /* 8f 44khz, 0f 48khz, 1f 96khz, 2f 192khz*/
+  /* otherwise if its 5.1 channels each nibble represents a channel group (G1, G2)*/
+  uint8_t  sampling_frequency; 
+  /* 00 1 channel, 01 2 channels, 11 5.1 channels*/
+  uint8_t  nr_channels;
+  uint8_t  unknown2;
+  uint8_t zero[10];
+} ATTRIBUTE_PACKED atsi_record_t;
+#define ATSI_RECORD_SIZE 16U
+
+/**
+ * ATS
+ *
+ * Structures relating to the Audio Title Set (ATS).
+ */
+
+/**
+ * Audio Title Set Information Management Table. Exclusive to DVD-Audio discs
+ */
+#define ATSI_RECORD_MAX_SIZE 8
+#define DOWNMIX_COEFF_MAX_SIZE 16
+
+typedef struct {
+  char          ats_identifier[12];
+  uint32_t      ats_last_sector; /* last sector of ATS_XX_0.BUP*/
+  uint8_t       zero_1[12];
+  uint32_t      atsi_last_sector; /* last sector of ATS_XX_0.IFO*/
+  uint16_t      specification_version;
+  uint32_t      unknown_1;
+  uint8_t       zero_2[90];
+  uint32_t      atsi_last_byte;
+  uint8_t       zero_3[60];
+  uint32_t      vtsm_vobs;       /* may be zeros */
+  uint32_t      atst_aobs;       /* atst or vtst */
+  uint32_t      vts_ptt_srpt;    /* may be zeros */
+  uint32_t      ats_pgci_ut;    /* sector */
+  uint32_t      vtsm_pgci_ut;    /* may be zeros */
+  uint32_t      vts_tmapt;       /* sector */
+  uint32_t      vtsm_c_adt;      /* sector */
+  uint32_t      vtsm_vobu_admap; /* sector */
+  uint32_t      vts_c_adt;       /* sector */
+  uint32_t      vts_vobu_admap;  /* sector */
+  uint8_t       zero_4[24];
+  /* the majority, or even all of these entries may be zero*/
+  atsi_record_t atsi_record[ATSI_RECORD_MAX_SIZE];
+  downmix_coeff_t downmix_coefficients[DOWNMIX_COEFF_MAX_SIZE];
+} ATTRIBUTE_PACKED atsi_mat_t;
+#define ATSI_MAT_SIZE 640U
+
+typedef struct {
+  uint16_t unknown_1; /* appears to be index, +0x100 for each iter*/
+  uint16_t unkown_2; /* either 0x0000 or 0x0100*/
+  uint32_t offset_record_table; 
+} ATTRIBUTE_PACKED atsi_title_index_t;
+#define ATSI_TITLE_INDEX_SIZE 8U
+
+/* this will be repeated for each track in each title*/
+typedef struct {
+  uint16_t unknown_1; /* will be 0x0000*/
+  uint16_t unknown_2; /* will be 0x0000*/
+  uint8_t  track_number_in_title;
+  uint8_t  unknown_3; /* will be 0x00*/
+  uint32_t first_pts_of_track; /* this is MPEG time, Not DVD time */
+
+  uint32_t length_pts_of_track; /* this is MPEG time, Not DVD time */
+
+  uint8_t  zero[6];
+} ATTRIBUTE_PACKED atsi_track_timestamp_t;
+#define ATSI_TRACK_TIMESTAMP_SIZE 20U
+
+/* this will come after all of the track timstamps, a set of 12byte sector pointer records. One for each track*/
+typedef struct {
+  uint32_t unknown_1; /* will be 0x01000000*/
+  uint32_t start_sector; /* relative to first AOB file*/
+  uint32_t end_sector; /* relative to first end of AOB file */
+} ATTRIBUTE_PACKED atsi_track_pointer_t;
+#define ATSI_TRACK_POINTER_SIZE 12U
+
+typedef struct {
+  uint16_t unknown_1; /* will be 0x0000*/
+  uint8_t  nr_tracks; /* unsure if this holds up for other files*/
+  uint8_t  nr_pointer_records; /* unsure if this holds up for other files*/
+  uint32_t length_pts; /* this is MPEG time, Not DVD time */
+
+  uint16_t unknown_3; /* will be 0x0000*/
+  uint16_t unknown_4; /* will be 0x0010*/
+  uint16_t start_sector_pointers_table; /* pointer to start of sector pointers table, relative to start of title record*/
+  uint16_t unknown_5; /* will be 0x0000*/
+  atsi_track_timestamp_t *atsi_track_timestamp_rows; /*length determined by nr_tracks*/
+  atsi_track_pointer_t   *atsi_track_pointer_rows;
+} ATTRIBUTE_PACKED atsi_title_record_t;
+#define ATSI_TITLE_ROW_TABLE_SIZE 16U
+
+typedef struct {
+  uint16_t               nr_titles;
+  uint16_t               zero_1;
+  uint32_t               last_byte_address;
+  atsi_title_index_t     *atsi_index_rows;   /* length determined by nr_titles*/
+  atsi_title_record_t    *atsi_title_row_tables;   /* length determined by nr_titles*/
+} ATTRIBUTE_PACKED atsi_title_table_t;
+#define ATSI_TITLE_TABLE_SIZE 8U
 
 /**
  * PartOfTitle Unit Information.
@@ -666,6 +944,7 @@ typedef struct {
   uint16_t pgcn;
   uint16_t pgn;
 } ATTRIBUTE_PACKED ptt_info_t;
+#define PTT_INFO_SIZE  4U
 
 /**
  * PartOfTitle Information.
@@ -674,6 +953,7 @@ typedef struct {
   uint16_t nr_of_ptts;
   ptt_info_t *ptt;
 } ATTRIBUTE_PACKED ttu_t;
+#define TTU_SIZE 2U
 
 /**
  * PartOfTitle Search Pointer Table.
@@ -729,29 +1009,61 @@ typedef struct {
  * VIDEO_TS.[IFO,BUP] file, and the VTSI, or Video Title Set Information, which
  * is read in from the VTS_XX_0.[IFO,BUP] files.
  */
+
+typedef enum{
+    IFO_UNKNOWN,
+    IFO_VIDEO,
+    IFO_AUDIO
+} ifo_format_t;
+
+/* format type will be used in order to reduce ammount of code refactoring, hopefully add some 
+ * detection mechanism later as well.
+ * */
 typedef struct {
-  dvd_file_t *file;
+  
+  
+  union{
+    struct{
+      /* VMGI */
+      vmgi_mat_t     *vmgi_mat;
+      tt_srpt_t      *tt_srpt;
+      pgc_t          *first_play_pgc;
+      ptl_mait_t     *ptl_mait;
+      vts_atrt_t     *vts_atrt;
+      txtdt_mgi_t    *txtdt_mgi;
 
-  /* VMGI */
-  vmgi_mat_t     *vmgi_mat;
-  tt_srpt_t      *tt_srpt;
-  pgc_t          *first_play_pgc;
-  ptl_mait_t     *ptl_mait;
-  vts_atrt_t     *vts_atrt;
-  txtdt_mgi_t    *txtdt_mgi;
+      /* Common */
+      pgci_ut_t      *pgci_ut;
+      c_adt_t        *menu_c_adt;
+      vobu_admap_t   *menu_vobu_admap;
 
-  /* Common */
-  pgci_ut_t      *pgci_ut;
-  c_adt_t        *menu_c_adt;
-  vobu_admap_t   *menu_vobu_admap;
+      /* VTSI */
+      vtsi_mat_t     *vtsi_mat;
+      vts_ptt_srpt_t *vts_ptt_srpt;
+      pgcit_t        *vts_pgcit;
+      vts_tmapt_t    *vts_tmapt;
+      c_adt_t        *vts_c_adt;
+      vobu_admap_t   *vts_vobu_admap;
+    };
+
+    struct{
+      /* SAMG */
+      samg_mat_t             *samg_mat;
+
+      /* AMGI */
+      amgi_mat_t             *amgi_mat;
+      tracks_info_table_t    *info_table_first_sector;
+      tracks_info_table_t    *info_table_second_sector;
+
+      /* ATSI */
+      atsi_mat_t             *atsi_mat;
+      atsi_title_table_t     *atsi_title_table;
+    };
+
+  };
+
+  ifo_format_t ifo_format;
 
-  /* VTSI */
-  vtsi_mat_t     *vtsi_mat;
-  vts_ptt_srpt_t *vts_ptt_srpt;
-  pgcit_t        *vts_pgcit;
-  vts_tmapt_t    *vts_tmapt;
-  c_adt_t        *vts_c_adt;
-  vobu_admap_t   *vts_vobu_admap;
 } ifo_handle_t;
 
 #endif /* LIBDVDREAD_IFO_TYPES_H */
